<?php

namespace App\Core;

use App\Services\AuthService;
use App\Utils\DarkMode; // Assurez-vous d'importer la classe DarkMode

/**
 * Simple routeur pour diriger les requêtes HTTP vers les contrôleurs appropriés.
 */
class Router
{
    private $routes = []; // Tableau pour stocker toutes les routes définies
    private $authService; // Service d'authentification pour vérifier l'accès aux routes protégées

    /**
     * Constructeur du routeur.
     * Initialise le service d'authentification.
     */
    public function __construct()
    {
        $this->authService = new AuthService(Database::getInstance());
    }

    /**
     * Ajoute une nouvelle route au routeur.
     *
     * @param string $method Méthode HTTP (GET, POST, etc.)
     * @param string $path Chemin de l'URL (ex: '/', '/dashboard')
     * @param string $controllerAction Action du contrôleur (ex: 'HomeController@index')
     * @param bool $requiresAuth Indique si la route nécessite une authentification (true par défaut)
     */
    public function addRoute($method, $path, $controllerAction, $requiresAuth = false)
    {
        $this->routes[] = [
            'method' => $method,
            'path' => $path,
            'controllerAction' => $controllerAction,
            'requiresAuth' => $requiresAuth
        ];
    }

    /**
     * Dispatche la requête entrante vers le contrôleur et l'action correspondants.
     * Gère également les redirections pour l'authentification et les erreurs 404.
     */
    public function dispatch()
    {
        // Récupère le chemin de l'URL demandé (sans les paramètres GET)
        $requestUri = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
        // Récupère la méthode HTTP de la requête (GET, POST, etc.)
        $requestMethod = $_SERVER['REQUEST_METHOD'];

        foreach ($this->routes as $route) {
            // Vérifie si la méthode et le chemin correspondent à une route définie
            if ($route['method'] === $requestMethod && $route['path'] === $requestUri) {
                // Si la route nécessite une authentification et que l'utilisateur n'est pas connecté
                if ($route['requiresAuth'] && !$this->authService->isLoggedIn()) {
                    // Redirige vers la page de connexion
                    header('Location: /login');
                    exit();
                }

                // Sépare le nom du contrôleur et de l'action
                list($controllerName, $actionName) = explode('@', $route['controllerAction']);
                // Construit le nom complet de la classe du contrôleur avec son namespace
                $controllerClass = "App\\Controllers\\" . $controllerName;

                // Vérifie si la classe du contrôleur existe
                if (class_exists($controllerClass)) {
                    // Instancie le contrôleur
                    $controller = new $controllerClass();
                    // Vérifie si la méthode de l'action existe dans le contrôleur
                    if (method_exists($controller, $actionName)) {
                        // Appelle la méthode de l'action
                        $controller->$actionName();
                        return; // Termine l'exécution après avoir traité la route
                    }
                }
            }
        }

        // Si aucune route correspondante n'est trouvée, retourne une erreur 404
        http_response_code(404);
        // Utilisation de DarkMode::getBodyClass() pour éviter l'erreur de syntaxe
        echo "<!DOCTYPE html><html lang=\"fr\"><head><meta charset=\"UTF-8\"><title>404 Non Trouvé</title><link rel=\"stylesheet\" href=\"/css/style.css\"></head><body class=\"" . DarkMode::getBodyClass() . "\">";
        echo "<div class=\"container\"><h1>404 Non Trouvé</h1>";
        echo "<p>La page que vous avez demandée n'a pas pu être trouvée.</p>";
        echo "<p><a href=\"/\">Retour à l'accueil</a></p></div></body></html>";
    }
}
